import com.github.scribejava.core.model.Response;

import java.io.IOException;
import java.net.URL;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Created by jhughes on 2/29/16.
 */

public class DataHandler implements TradeKingReceiver, Runnable{
    private static final String weeklyOptionsUrl = "http://www.cboe.com/publish/scheduledtask/mktdata/cboesymboldirweeklys.csv";
    private static final String nyseUrl = "http://www.nasdaq.com/screening/companies-by-name.aspx?letter=0&exchange=nyse&render=download";
    private static final String nasdaqUrl = "http://www.nasdaq.com/screening/companies-by-name.aspx?letter=0&exchange=nasdaq&render=download";
    private static final String sp500Url = "https://raw.githubusercontent.com/datasets/s-and-p-500-companies/master/data/constituents.csv";
    public static ConcurrentHashMap<String,Stock> allStocks;
    public static ConcurrentHashMap<String, Integer> allSectors;
    private static final String USER_AGENT = "Mozilla/5.0";
    private TradeKingClient tc;
    private TradeKingParser parse;


    public DataHandler(){
        allStocks = new ConcurrentHashMap<>();
        allSectors = new ConcurrentHashMap<>();
        tc = new TradeKingClient();
        parse = new TradeKingParser();
    }

    public BufferedReader getUrl(String url) {
        BufferedReader in = null;
        try {
            URL obj = new URL(url);
            HttpURLConnection con = (HttpURLConnection) obj.openConnection();

            // optional default is GET
            con.setRequestMethod("GET");

            //add request header
            con.setRequestProperty("User-Agent", USER_AGENT);

            int responseCode = con.getResponseCode();
            System.out.println("\nSending 'GET' request to URL : " + url);
            System.out.println("Response Code : " + responseCode);

            in = new BufferedReader(
                    new InputStreamReader(con.getInputStream()));
        } catch (Exception e) {
            System.out.println("Unable to connect to url:" + e);
        }
        return in;
    }

    public void getWeeklyList(){
            BufferedReader in = getUrl(weeklyOptionsUrl);
            try {

                String inputLine;
                //ignore our first couple of lines
                inputLine = in.readLine();
                inputLine = in.readLine();

                allStocks.clear();
                while ((inputLine = in.readLine()) != null) {
                    String[] weekly = inputLine.split(",");
                    allStocks.put(weekly[1], new Stock(weekly[1], weekly[0]));
                }
                in.close();

                //print result
                System.out.println("Number of weekly optionable stocks: " + allStocks.size());
            }
            catch (IOException e){
                System.out.println("Exception getting weekly list:" + e);
            }
    }

    public void getSP500(){
        BufferedReader in = getUrl(sp500Url);
        try {

            String inputLine;
            //ignore our first couple of lines
            inputLine = in.readLine();

            allStocks.clear();
            while ((inputLine = in.readLine()) != null) {
                String[] sp500 = inputLine.split(",");
                allStocks.put(sp500[0], new Stock(sp500[0], sp500[1]));
                Stock s= allStocks.get(sp500[0]);
                if(sp500[2].equals(" Inc.\"")){
                    s.setSector(sp500[3]);
                    addSector(sp500[3]);
                }else {
                    s.setSector(sp500[2]);
                    addSector(sp500[2]);
                }
            }
            in.close();

            //print result
            System.out.println("Number of SP 500 stocks: " + allStocks.size());
            for (String s: allSectors.keySet()){
                System.out.println(s + ": " + allSectors.get(s));
            }

        }
        catch (IOException e){
            System.out.println("Exception getting weekly list:" + e);
        }
    }

    public void addSector(String key){
        if(!allSectors.containsKey(key)){
            allSectors.put(key, 1);
        }else{
            allSectors.put(key, allSectors.get(key) + 1);
        }
    }

    public void getSectors(){
        BufferedReader in1 = getUrl(nasdaqUrl);
        BufferedReader in2 = getUrl(nyseUrl);

        HashMap<String, String> sectorList = new HashMap<>();

        try {

            String inputLine;
            inputLine = in1.readLine();

            while ((inputLine = in1.readLine()) != null) {
                String[] weekly = inputLine.split("\",");
                weekly[0] = weekly[0].replace("\"", "");
                if(allStocks.containsKey(weekly[0])){
                    Stock s= allStocks.get(weekly[0]);
                    s.setSector(weekly[5]);
                    addSector(weekly[5]);
                    System.out.println("Stock: " + s + " Sector: " + weekly[5].replace("\"", ""));
                }
            }
            in1.close();

            inputLine = in2.readLine();
            while ((inputLine = in2.readLine()) != null) {
                String[] weekly = inputLine.split("\",");
                weekly[0] = weekly[0].replace("\"", "");
                if(allStocks.containsKey(weekly[0])) {
                    Stock s = allStocks.get(weekly[0]);
                    s.setSector(weekly[5]);
                    addSector(weekly[5]);
                    System.out.println("Stock: " + s + " Sector: " + weekly[5].replace("\"", ""));
                }
            }
            in2.close();

        }
        catch (IOException e){
            System.out.println("Exception getting weekly list:" + e);
        }
    }

    public void run(){
        //getWeeklyList();
        getSP500();
        //getSectors();
        while(true) {
            updateStocks();
        }

    }

    public void updateStocks(){
       for(Stock s: allStocks.values()){
           tc.getQuotes(s.getSymbol(), this);
           try {
               Thread.sleep(50);
           }catch(InterruptedException e){
               System.out.println("Could not sleep: "+e);
           }
       }
    }

    @Override
    public void receiveResponse(TradeKingRequest request, Response response) {

        System.out.println(response.getBody());
        switch(request.getRequestType()){
            case QUOTE: parse.parseQuote(response);

            default: break;
        }
    }
}
