/**
 * Created by jhughes on 3/1/16.
 */

import processing.core.*;
import processing.opengl.PShader;
import processing.opengl.PGraphicsOpenGL;
import peasy.*;

import javax.activation.*;
import java.math.BigDecimal;
import java.math.MathContext;
import java.util.*;


import java.math.RoundingMode;
import java.util.concurrent.ConcurrentHashMap;

public class Sketch extends PApplet{

    private PShader toon;
    private PShader rainbow;
    private PShader shadow;
    private PShader basicSphere;
    private PeasyCam cam;
    private float rDelta = 0.1f;
    private float theta = PI ;
    private float phi = random(0, 2*PI);
    private float radius = 80.0f;
    private float increment = 0.0f;
    private HashMap<String, StockSphere> renderSpheres;
    private PFont helvetica;
    private HashMap<String, BigDecimal> sectorMarketCaps;
    private HashMap<String, BigDecimal> sectorChanges;
    private HashMap<String, BigDecimal> sectorVolumes;
    private Boolean pause = false;
    private StockSphere displaySphere = null;



    public void settings() {
        size(1280, 720, P3D);
        pixelDensity(2);

    }

    public void setup(){
        renderSpheres = new HashMap<>();
        sectorMarketCaps = new HashMap<>();
        sectorChanges = new HashMap<>();
        sectorVolumes = new HashMap<>();
        noStroke();
        fill(204);
        toon = loadShader("SphereFrag.glsl", "SphereVert.glsl");
        toon.set("fraction", 1.0f);
        rainbow = loadShader("RainbowFrag.glsl");
        rainbow.set("viewport",640.0f, 480.0f);
        rainbow.set("time", millis()/ 1000.0f);
        shadow = loadShader("SphereShadows.glsl");
        shadow.set("viewport",640.0f, 480.0f);

        basicSphere = loadShader("BasicSphere.glsl");
        basicSphere.set("viewport", 1280.0f, 720.0f);

        cam = new PeasyCam(this, 100);
        cam.setMinimumDistance(5);
        cam.setMaximumDistance(10000);
        colorMode(HSB);
        helvetica = createFont("Helvetica-Light", 10);
        textFont(helvetica);
        textMode(SHAPE);

    }



    public BigDecimal getMinMarketCap(){
          Stock s = Collections.min(DataHandler.allStocks.values(), new Comparator<Stock>() {
              @Override
              public int compare(Stock o1, Stock o2) {
                  if (o1.getMarketCap() != null && o2.getMarketCap() != null) {
                      return o1.getMarketCap().compareTo(o2.getMarketCap());
                  }
                  return 0;
              }
          });
        return s.getMarketCap();
    }

    public BigDecimal getMaxMarketCap(){
        Stock s = Collections.max(DataHandler.allStocks.values(), new Comparator<Stock>() {
            @Override
            public int compare(Stock o1, Stock o2) {
                if (o1.getMarketCap() != null && o2.getMarketCap() != null) {
                    return o1.getMarketCap().compareTo(o2.getMarketCap());
                }
                return 0;
            }
        });
        return s.getMarketCap();
    }

    public BigDecimal getMinRelativeVolume(){
        Stock s = Collections.min(DataHandler.allStocks.values(), new Comparator<Stock>() {
            @Override
            public int compare(Stock o1, Stock o2) {
                if (o1.getVolume() != null && o2.getVolume() != null) {
                    return o1.getVolume().divide(o1.getAdv_21(), 2, RoundingMode.HALF_UP)
                            .compareTo(o2.getVolume()
                                    .divide(o2.getAdv_21(), 2, RoundingMode.HALF_UP));
                }
                return 0;
            }
        });
        return s.getVolume().divide(s.getAdv_21(), 2, RoundingMode.HALF_UP);
    }

    public BigDecimal getMaxRelativeVolume(){
        Stock s = Collections.max(DataHandler.allStocks.values(), new Comparator<Stock>() {
            @Override
            public int compare(Stock o1, Stock o2) {
                if (o1.getVolume() != null && o2.getVolume() != null) {
                    return o1.getVolume().divide(o1.getAdv_21(), 2, RoundingMode.HALF_UP)
                            .compareTo(o2.getVolume()
                                    .divide(o2.getAdv_21(), 2, RoundingMode.HALF_UP));
                }
                return 0;
            }
        });
        return s.getVolume().divide(s.getAdv_21(), 2, RoundingMode.HALF_UP);
    }

    public ArrayList<Stock> relativeVolumeList(){
        ArrayList<Stock> sortedList = new ArrayList<>(DataHandler.allStocks.values());
        Collections.sort(sortedList, new Comparator<Stock>() {
            @Override
            public int compare(Stock o1, Stock o2) {
                if (o1.getVolume() != null && o2.getVolume() != null) {
                    return o1.getVolume().divide(o1.getAdv_21(), 2, RoundingMode.HALF_UP)
                            .compareTo(o2.getVolume()
                                    .divide(o2.getAdv_21(), 2, RoundingMode.HALF_UP));
                }else if(o1.getVolume() != null){
                    return 1;
                }else if(o2.getVolume() !=null){
                    return  -1;
                }else {
                    return 0;
                }
            }
        });
        return sortedList;
    }

    public void clearSectorMarketCaps(){
        if(DataHandler.allSectors != null) {
            for (String s : DataHandler.allSectors.keySet()) {
                sectorMarketCaps.put(s, BigDecimal.ZERO);
            }
        }
    }

    public StockSphere createSphere(Stock s){

        StockSphere ss = new StockSphere(random(0.0f, 2.0f * PI), random(0.0f, PI), random(80.0f, 100.0f), random(0.01f, 0.10f), 2.0f, s);
        renderSpheres.put(s.getSymbol(), ss);
        return ss;

    }

    public void translateSphere(StockSphere s, float x, float y, float z){
        translate(x, y, z);
        s.translateSphere(x, y, z);
    }

    public void clearSectorChanges(){
        if(DataHandler.allSectors != null) {
            for (String s : DataHandler.allSectors.keySet()) {
                sectorChanges.put(s, BigDecimal.ZERO);
            }
        }
    }

    public void clearSectorVolumes(){
        if(DataHandler.allSectors != null) {
            for (String s : DataHandler.allSectors.keySet()) {
                sectorVolumes.put(s, BigDecimal.ZERO);
            }
        }
    }


    public void drawSphere(StockSphere s, int sphereIndex){
        pushMatrix();
        colorMode(HSB);
        if(DataHandler.allSectors.keySet().size() != 0){
            int c = color((int) (255 / DataHandler.allSectors.keySet().size()) * sphereIndex, 255, 255, 255);
            fill(c);
        }
        //translate(radius * sin(theta + increment) * cos(phi), radius * sin(theta + increment) * sin(phi), radius * cos(theta + increment));

        translateSphere(s, s.getRadius() * sin(s.getTheta() /*+ s.getSpeed()*/) * cos(s.getPhi()),
                s.getRadius() * sin(s.getTheta()/* + s.getSpeed()*/) * sin(s.getPhi()),
                s.getRadius() * cos(s.getTheta() /*+ s.getSpeed()*/));

        translateSphere(s, 150.0f * sphereIndex, -sectorChanges.get(s.getUnderlying().getSector()).floatValue() * 1000,
                sectorVolumes.get(s.getUnderlying().getSector()).floatValue() / 1000);
        sphere(s.getSize());
        colorMode(RGB);
        popMatrix();

    }

    public void lightsOn(){
        ambientLight(102, 102, 102);
        lightSpecular(204, 204, 204);
        specular(255, 255, 255);
        directionalLight(102, 102, 102, 0, 1, 0);
        shininess(1.0f);
    }

    public void addSignature(){
        noLights();
        smooth();
        textSize(12);
        colorMode(RGB);
        stroke(200);
        text("The S&P 500",width - 100, height - 48, 0);
        text("Jordan Hughes",width - 100, height - 36, 0);
        text("Computer Science, MS", width - 100, height - 24);
        text("MAT 259, 2016",width - 100, height - 12, 0);
        //textMode(MODEL);
        //colorMode(HSB);
        lightsOn();
    }


    public void draw(){
        clearSectorMarketCaps();
        clearSectorChanges();
        clearSectorVolumes();
        background(0);
        noStroke();
        sphereDetail(18);
        lightsOn();

        ArrayList<String> sectors = new ArrayList<String>(DataHandler.allSectors.keySet());
        //Integer pctChangCt = 0, volCount = 0;

        for(Stock s: DataHandler.allStocks.values()){
            if(s.getPctchg() != null){
                sectorChanges.put(s.getSector(), sectorChanges.get(s.getSector()).add(s.getPctchg()));
            }
            if(s.getVolume() != null){
                sectorVolumes.put(s.getSector(), sectorVolumes.get(s.getSector()).add(s.getVolume()));
            }
        }

        for(String s: sectors) {
            //average out the volumes and percent changes
            try {
                sectorChanges.put(s, sectorChanges.get(s).divide(new BigDecimal(DataHandler.allSectors.get(s)), 2, RoundingMode.HALF_UP));
                sectorVolumes.put(s, sectorVolumes.get(s).divide(new BigDecimal(DataHandler.allSectors.get(s)), 2, RoundingMode.HALF_UP));
            } catch (Exception e) {
                System.out.println(e);
            }
        }


        for(Stock s: DataHandler.allStocks.values()){
            if(!renderSpheres.containsKey(s.getSymbol())){
                renderSpheres.put(s.getSymbol(), createSphere(s));
            }
            if(renderSpheres.get(s.getSymbol()).getUnderlying().getMarketCap() != null){
                sectorMarketCaps.put(s.getSector(), sectorMarketCaps.get(s.getSector()).add(s.getMarketCap()));
               // sectorChanges.put(s.getSector(), sectorChanges.get(s.getSector()).add(s.getPctchg()));
                StockSphere newSphere = renderSpheres.get(s.getSymbol());
                newSphere.clearPos();
                float size = map(newSphere.getUnderlying().getMarketCap().floatValue(), getMinMarketCap().floatValue(),
                        getMaxMarketCap().floatValue(), 1.0f, 20.0f);
                newSphere.setSize(size);


                float speed = map(newSphere.getUnderlying().getVolume().divide(newSphere.getUnderlying().getAdv_21(), 2, RoundingMode.HALF_UP).floatValue(),
                        getMinRelativeVolume().floatValue(),
                        getMaxRelativeVolume().floatValue(),
                        0.01f, 0.50f);
                newSphere.setSpeed(speed);
                drawSphere(renderSpheres.get(s.getSymbol()), sectors.indexOf(s.getSector()));
                if(!pause) {
                    renderSpheres.get(s.getSymbol()).incrementPosition();
                }
            }

        }

        Integer i = 0;
        for(String s: sectors){
            stroke(255);
            strokeWeight(1.5f);
            noLights();
            noFill();
            Integer j = 0;
            for(String s2: sectors) {
               /* if(sectorChanges.get(s).compareTo(sectorChanges.get(s2)) == 1){
                    stroke(color(0, 255, 0));
                }else{
                    stroke(color(255, 0, 0));
                }*/
                if(!s.equals(s2)) {
                    bezier(150.0f * i, sectorChanges.get(s).floatValue() * -1000, sectorVolumes.get(s).floatValue() / 1000,
                            (150.0f * i) + 300.f, sectorChanges.get(s).floatValue() * 1000, sectorVolumes.get(s).floatValue() / 1000,
                            (150.0f * j) + 300.f, sectorChanges.get(s2).floatValue() * 1000, sectorVolumes.get(s2).floatValue() / 1000,
                            150.0f * j, sectorChanges.get(s2).floatValue() * -1000, sectorVolumes.get(s2).floatValue() / 1000
                    );
                }
                j++;
            }
            noStroke();
            lightsOn();
            colorMode(HSB);
            int c = color((int) (255 / DataHandler.allSectors.keySet().size()) * i, 255, 255, 255);
            fill(c);
            pushMatrix();
            translate(150.0f * i, sectorChanges.get(s).floatValue()*-1000, sectorVolumes.get(s).floatValue() / 1000);
            float size = map(sectorMarketCaps.get(s).floatValue(), Collections.min(sectorMarketCaps.values()).floatValue(),
                    Collections.max(sectorMarketCaps.values()).floatValue(), 20.0f, 60.0f);
            sphere(size);
            popMatrix();
            colorMode(RGB);
            noLights();
            textSize(13);
            fill(255);
            textAlign(CENTER, CENTER);
            text(s, 150.0f * i, sectorChanges.get(s).floatValue()*-1000 - 100.0f, sectorVolumes.get(s).floatValue() / 1000);

            if(displaySphere != null){
                if(displaySphere.getPos().get(0) != 0 && displaySphere.getPos().get(1) != 0 && displaySphere.getPos().get(2) != 0) {
                    if(displaySphere.getUnderlying() != null) {
                    text(displaySphere.getUnderlying().getSymbol() + "", displaySphere.getPos().get(0), displaySphere.getPos().get(1) + -30.0f, displaySphere.getPos().get(2));
                    fill(155);
                    textSize(10);
                    text(displaySphere.getUnderlying().getName(), displaySphere.getPos().get(0), displaySphere.getPos().get(1) + -20.0f, displaySphere.getPos().get(2));

                    if(displaySphere.getUnderlying().getPctchg() != null) {
                        if (displaySphere.getUnderlying().getPctchg().compareTo(BigDecimal.ZERO) == 1) {
                            fill(color(0, 255, 0));
                            text("+" + displaySphere.getUnderlying().getPctchg().round(new MathContext(2, RoundingMode.HALF_UP)).toString() + "%", displaySphere.getPos().get(0), displaySphere.getPos().get(1) + -10.0f, displaySphere.getPos().get(2));
                        } else {
                            fill(color(255, 0, 0));
                            text(displaySphere.getUnderlying().getPctchg().round(new MathContext(2, RoundingMode.HALF_UP)).toString() + "%", displaySphere.getPos().get(0), displaySphere.getPos().get(1) + -10.0f, displaySphere.getPos().get(2));
                        }
                    }

                    }
                }
            }

            lightsOn();
            i++;
        }
        rDelta = rDelta + 0.01f;
        increment = increment + 0.05f;

        cam.beginHUD();
        addSignature();
        cam.endHUD();

    }

    public void keyPressed(){
        if(key == 'p' || key == 'P'){
            pause = !pause;
        }
        if(key == 'a' || key == 'A'){
           // pause = true;

            //renderSpheres.get("AAPL").getPo
            //Collections.reverse(relativeVolumeList());
            ArrayList<Stock> volList = relativeVolumeList();
            Collections.reverse(volList);
            for (Stock v: volList){
                if(v.getVolume() != null) {
                    System.out.println(v.getSymbol() + ": " + v.getVolume().divide(v.getAdv_21(), 2, RoundingMode.HALF_UP));
                }
            }

            Timer timer = new Timer();
            Timer timer2 = new Timer();
            Timer timer3 = new Timer();
          //  for(int i=0; i < 10; i ++){

                timer.schedule(new TimerTask() {
                    @Override
                    public void run() {
                        if(displaySphere != null){
                            displaySphere.setPaused(false);
                        }
                        StockSphere s = renderSpheres.get(volList.get(0).getSymbol());
                        displaySphere = s;
                        renderSpheres.get(volList.get(0).getSymbol()).setPaused(true);
                        cam.lookAt(renderSpheres.get(volList.get(0).getSymbol()).getPos().get(0),
                                renderSpheres.get(volList.get(0).getSymbol()).getPos().get(1),
                                renderSpheres.get(volList.get(0).getSymbol()).getPos().get(2), 200, 4000);
                    }
                }, 6000 * 0);

            timer.schedule(new TimerTask() {
                @Override
                public void run() {
                    if(displaySphere != null){
                        displaySphere.setPaused(false);
                    }
                    StockSphere s = renderSpheres.get(volList.get(1).getSymbol());
                    displaySphere = s;
                    renderSpheres.get(volList.get(1).getSymbol()).setPaused(true);
                    cam.lookAt(renderSpheres.get(volList.get(1).getSymbol()).getPos().get(0),
                            renderSpheres.get(volList.get(1).getSymbol()).getPos().get(1),
                            renderSpheres.get(volList.get(1).getSymbol()).getPos().get(2), 200, 4000);
                }
            }, 6000 * 1);

            timer.schedule(new TimerTask() {
                @Override
                public void run() {
                    if(displaySphere != null){
                        displaySphere.setPaused(false);
                    }
                    StockSphere s = renderSpheres.get(volList.get(2).getSymbol());
                    displaySphere = s;
                    renderSpheres.get(volList.get(2).getSymbol()).setPaused(true);
                    cam.lookAt(renderSpheres.get(volList.get(2).getSymbol()).getPos().get(0),
                            renderSpheres.get(volList.get(2).getSymbol()).getPos().get(1),
                            renderSpheres.get(volList.get(2).getSymbol()).getPos().get(2), 200, 4000);
                }
            }, 6000 * 2);

            timer.schedule(new TimerTask() {
                @Override
                public void run() {
                    if(displaySphere != null){
                        displaySphere.setPaused(false);
                    }
                    displaySphere = null;
                }
            }, 6000 * 3);
           // }
        }
    }
}
