
import com.github.scribejava.core.oauth.OAuthService;
import com.github.scribejava.core.builder.*;
import com.github.scribejava.core.model.*;
import com.github.scribejava.core.oauth.*;
import java.util.Queue;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.LinkedBlockingQueue;


/**
 * Created by jhughes on 2/29/16.
 */
public class TradeKingClient {

    private static OAuth10aService service;
    private static OAuth1AccessToken accessToken;

    private static final String CONSUMER_KEY = "";
    private static final String CONSUMER_SECRET = "";
    private static final String OAUTH_TOKEN = "";
    private static final String OAUTH_TOKEN_SECRET = "";

    private static final String PROTECTED_RESOURCE_URL = "https://api.tradeking.com/v1/member/profile.json";
    private static final String BASE_URL = "https://api.tradeking.com/v1/";
    private static final String DATA_FORMAT = ".json";
    private static final String STRIKES_URL = "market/options/strikes";
    private static final String QUOTES_URL = "market/ext/quotes";
    private static final String CLOCK_URL = "market/clock";
    private static final String SEARCH_URL = "market/options/search";


    private LinkedBlockingQueue<TradeKingRequest> requestQueue;
    private Timer queueTimer;
    private static final int REQUEST_DELAY = 1000;


    public TradeKingClient(){
        service = new ServiceBuilder()
                .apiKey(CONSUMER_KEY)
                .apiSecret(CONSUMER_SECRET)
                .build(new TradeKingApi());

        accessToken = new OAuth1AccessToken(OAUTH_TOKEN, OAUTH_TOKEN_SECRET);
        queueTimer = new Timer();
        queueTimer.schedule(new TimerTask() {
            @Override
            public void run() {
                checkQueue();
            }
        }, REQUEST_DELAY, REQUEST_DELAY);
        requestQueue = new LinkedBlockingQueue<>() ;



    }

    public void checkQueue(){
     if(!requestQueue.isEmpty()){
         TradeKingRequest request = requestQueue.remove();
         OAuthRequest oAuthRequest = request.getRequest();
         service.signRequest(accessToken, oAuthRequest);
         Response response = oAuthRequest.send();
         request.getReceiver().receiveResponse(request, response);
     }
    }

    public void getStrikes(String symbol, TradeKingReceiver receiver){
        OAuthRequest request = new OAuthRequest(Verb.GET, BASE_URL+STRIKES_URL+DATA_FORMAT+"?symbol="+symbol, service);
        TradeKingRequest newRequest = new TradeKingRequest(receiver, request, TradeKingRequest.TradeKingRequestType.STRIKES);
        requestQueue.add(newRequest);
    }

    public void getQuotes(String symbol, TradeKingReceiver receiver){
        OAuthRequest request = new OAuthRequest(Verb.GET, BASE_URL+QUOTES_URL+DATA_FORMAT+"?symbols="+symbol, service);
        TradeKingRequest newRequest = new TradeKingRequest(receiver, request, TradeKingRequest.TradeKingRequestType.QUOTE);
        requestQueue.add(newRequest);
    }

    public void getClock(TradeKingReceiver receiver){
        OAuthRequest request = new OAuthRequest(Verb.GET, BASE_URL+CLOCK_URL+DATA_FORMAT, service);
        TradeKingRequest newRequest = new TradeKingRequest(receiver, request, TradeKingRequest.TradeKingRequestType.CLOCK);
        requestQueue.add(newRequest);
    }

    public void getOptions(String symbol, boolean CallPut, TradeKingReceiver receiver){
        String query = "";
        if(CallPut) {
            query = "&query=put_call-eq%3Acall";
        }else{
            query = "&query=put_call-eq%3Aput";
        }
        OAuthRequest request = new OAuthRequest(Verb.GET, BASE_URL+SEARCH_URL+DATA_FORMAT+"?symbol="+symbol+query, service);
        TradeKingRequest newRequest = new TradeKingRequest(receiver, request, TradeKingRequest.TradeKingRequestType.STRIKES);
        requestQueue.add(newRequest);
    }
}
