//If you're new to GLSL or programming in general,
//I encourage you to play with these variables and values to see what they do!

#ifdef GL_ES
precision mediump float;
precision mediump int;
#endif

#define PROCESSING_COLOR_SHADER

//uniform mat4 transform;
uniform vec2 viewport;
uniform float time;

//Variable declarations

//Sets background colour(red, green, blue)
vec3 bgCol = vec3(1.0, 1.0, 1.0);

//Sets size of the sphere and brightness of the shine
float sphereScale = 0.5;
float sphereShine = 0.5;

//Sets diffuse colour(red, green, blue), specular colour(red, green, blue),
//and initial specular point position(x, y)
vec3 sphereDiff = vec3(0.5, 0.0, 0.5);
vec3 sphereSpec = vec3(1.0, 1.0, 1.0);
vec2 specPoint = vec2(0.2, -0.1);

//Main method/function



void main() {
	//Creates shader pixel coordinates
	vec2 uv = gl_FragCoord.xy / viewport.xy;

	//Sets the position of the camera
	//vec2 p = uv * 2.3 - 1.0;
	vec2 p = uv * 10;
	p.x *= viewport.x / viewport.y;

	//Rotates the sphere in a circle
	p.x += cos(-time) * 0.35;
	p.y += sin(-time) * 0.35;

	//Rotates the specular point with the sphere
	specPoint.x += cos(-time) * 0.35;
	specPoint.y += sin(-time) * 0.35;

	//Sets the radius of the sphere to the middle of the screen
	float radius = sqrt(dot(p, p));

	vec3 col = bgCol;

	//Sets the initial dark shadow around the edge of the sphere
	float f = smoothstep(sphereScale * 0.9, sphereScale, length(p + specPoint));
	col -= mix(col, vec3(0.0), f) * 0.2;

	//Only carries out the logic if the radius of the sphere is less than the scale
	if(radius < sphereScale) {
		vec3 bg = col;

		//Sets the diffuse colour of the sphere (solid colour)
		col = sphereDiff;

		//Adds smooth dark borders to help achieve 3D look
		f = smoothstep(sphereScale * 0.7, sphereScale, radius);
		col = mix(col, sphereDiff * 0.45, f);

		//Adds specular glow to help achive 3D look
		f = 1.0 - smoothstep(-0.2, 0.6, length(p - specPoint));
		col += f * sphereShine * sphereSpec;

		//Smoothes the edge of the sphere
		f = smoothstep(sphereScale - 0.01, sphereScale, radius);
		col = mix(col, bg, f);
	}

	//The final output of the shader logic above
	//fragColor is a vector with 4 paramaters(red, green, blue, alpha)
	//Only 2 need to be used here, as "col" is a vector that already carries r, g, and b values
	gl_FragColor = vec4(col, 1);
}
